@@ -19,9 +19,11 @@ import os
 import javabridge
 import logging
 import argparse
+import traceback
 import weka.core.jvm as jvm
 import weka.core.types as arrays
 import weka.core.classes as classes
+import weka.core.serialization as serialization
 from numpy import *
 from weka.core.classes import JavaObject, join_options, OptionHandler, Random, SelectedTag, Tags, Tag, JavaArray, \
     is_instance_of
@@ -221,6 +223,44 @@ class Classifier(OptionHandler):
                 "weka/classifiers/AbstractClassifier", "makeCopy",
                 "(Lweka/classifiers/Classifier;)Lweka/classifiers/Classifier;", classifier.jobject))
 
+    @classmethod
+    def deserialize(cls, ser_file):
+        """
+        Deserializes a classifier from a file.
+
+        :param ser_file: the model file to deserialize
+        :type ser_file: str
+        :return: model and, if available, the dataset header
+        :rtype: tuple
+        """
+
+        objs = serialization.read_all(ser_file)
+        if len(objs) == 1:
+            return Classifier(jobject=objs[0]), None
+        elif len(objs) == 2:
+            return Classifier(jobject=objs[0]), Instances(jobject=objs[1])
+        else:
+            raise Exception(
+                "Excepted one or two objects in the model file (%s), but encountered: %d" % (ser_file, len(objs)))
+
+    def serialize(self, ser_file, header=None):
+        """
+        Serializes the classifier to the specified file.
+
+        :param ser_file: the file to save the model to
+        :type ser_file: str
+        :param header: the (optional) dataset header to store alongside; recommended
+        :type header: Instances
+        """
+
+        if (header is not None) and header.num_instances > 0:
+            header = Instances.template_instances(header)
+
+        if header is not None:
+            serialization.write_all(ser_file, [self, header])
+        else:
+            serialization.write(ser_file, self)
+
 
 class SingleClassifierEnhancer(Classifier):
     """
@@ -2114,11 +2154,15 @@ def predictions_to_instances(data, preds):
     return result
 
 
-def main():
+def main(args=None):
     """
     Runs a classifier from the command-line. Calls JVM start/stop automatically.
     Use -h to see all options.
+
+    :param args: the command-line arguments to use, uses sys.argv if None
+    :type args: list
     """
+
     parser = argparse.ArgumentParser(
         description='Performs classification/regression from the command-line. Calls JVM start/stop automatically.')
     parser.add_argument("-j", metavar="classpath", dest="classpath", help="additional classpath, jars/directories")
@@ -2138,7 +2182,7 @@ def main():
     parser.add_argument("-g", metavar="graph", dest="graph", help="output file for graph (if supported)")
     parser.add_argument("classifier", help="classifier classname, e.g., weka.classifiers.trees.J48")
     parser.add_argument("option", nargs=argparse.REMAINDER, help="additional classifier options")
-    parsed = parser.parse_args()
+    parsed = parser.parse_args(args)
     jars = []
     if parsed.classpath is not None:
         jars = parsed.classpath.split(os.pathsep)
@@ -2179,13 +2223,31 @@ def main():
         if len(parsed.option) > 0:
             classifier.options = parsed.option
         print(Evaluation.evaluate_model(classifier, params))
-    except Exception, e:
-        print(e)
+    except Exception:
+        print(traceback.format_exc())
     finally:
         jvm.stop()
 
+
+def sys_main():
+    """
+    Runs the main function using the system cli arguments, and
+    returns a system error code.
+
+    :return: 0 for success, 1 for failure.
+    :rtype: int
+    """
+
+    try:
+        main()
+        return 0
+    except Exception:
+        print(traceback.format_exc())
+        return 1
+
+
 if __name__ == "__main__":
     try:
         main()
-    except Exception, ex:
-        print(ex)
+    except Exception:
+        print(traceback.format_exc())
