@@ -12,19 +12,22 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # clusterers.py
-# Copyright (C) 2014-2015 Fracpete (pythonwekawrapper at gmail dot com)
+# Copyright (C) 2014-2019 Fracpete (pythonwekawrapper at gmail dot com)
 
 import javabridge
 import logging
 import os
 import sys
 import argparse
+import traceback
 import weka.core.jvm as jvm
 import weka.core.classes as classes
+import weka.core.serialization as serialization
 from weka.core.classes import JavaObject, join_options
 from weka.core.classes import OptionHandler
 from weka.core.classes import Random
 from weka.core.capabilities import Capabilities
+from weka.core.dataset import Instances
 from weka.filters import Filter
 
 # logging setup
@@ -170,6 +173,44 @@ class Clusterer(OptionHandler):
                 "weka/clusterers/AbstractClusterer", "makeCopy",
                 "(Lweka/clusterers/Clusterer;)Lweka/clusterers/Clusterer;", clusterer.jobject))
 
+    @classmethod
+    def deserialize(cls, ser_file):
+        """
+        Deserializes a clusterer from a file.
+
+        :param ser_file: the model file to deserialize
+        :type ser_file: str
+        :return: model and, if available, the dataset header
+        :rtype: tuple
+        """
+
+        objs = serialization.read_all(ser_file)
+        if len(objs) == 1:
+            return Clusterer(jobject=objs[0]), None
+        elif len(objs) == 2:
+            return Clusterer(jobject=objs[0]), Instances(jobject=objs[1])
+        else:
+            raise Exception(
+                "Excepted one or two objects in the model file (%s), but encountered: %d" % (ser_file, len(objs)))
+
+    def serialize(self, ser_file, header=None):
+        """
+        Serializes the clusterer to the specified file.
+
+        :param ser_file: the file to save the model to
+        :type ser_file: str
+        :param header: the (optional) dataset header to store alongside; recommended
+        :type header: Instances
+        """
+
+        if (header is not None) and header.num_instances > 0:
+            header = Instances.template_instances(header)
+
+        if header is not None:
+            serialization.write_all(ser_file, [self, header])
+        else:
+            serialization.write(ser_file, self)
+
 
 class SingleClustererEnhancer(Clusterer):
     """
@@ -380,11 +421,15 @@ class ClusterEvaluation(JavaObject):
             clusterer.jobject, data.jobject, num_folds, rnd.jobject)
 
 
-def main():
+def main(args=None):
     """
     Runs a clusterer from the command-line. Calls JVM start/stop automatically.
     Use -h to see all options.
+
+    :param args: the command-line arguments to use, uses sys.argv if None
+    :type args: list
     """
+
     parser = argparse.ArgumentParser(
         description='Performs clustering from the command-line. Calls JVM start/stop automatically.')
     parser.add_argument("-j", metavar="classpath", dest="classpath", help="additional classpath, jars/directories")
@@ -400,7 +445,7 @@ def main():
     parser.add_argument("-g", metavar="graph", dest="graph", help="graph output file (if supported)")
     parser.add_argument("clusterer", help="clusterer classname, e.g., weka.clusterers.SimpleKMeans")
     parser.add_argument("option", nargs=argparse.REMAINDER, help="additional clusterer options")
-    parsed = parser.parse_args()
+    parsed = parser.parse_args(args)
     jars = []
     if parsed.classpath is not None:
         jars = parsed.classpath.split(os.pathsep)
@@ -433,13 +478,31 @@ def main():
         if len(parsed.option) > 0:
             clusterer.options = parsed.option
         print(ClusterEvaluation.evaluate_clusterer(clusterer, params))
-    except Exception, e:
-        print(e)
+    except Exception:
+        print(traceback.format_exc())
     finally:
         jvm.stop()
 
+
+def sys_main():
+    """
+    Runs the main function using the system cli arguments, and
+    returns a system error code.
+
+    :return: 0 for success, 1 for failure.
+    :rtype: int
+    """
+
+    try:
+        main()
+        return 0
+    except Exception:
+        print(traceback.format_exc())
+        return 1
+
+
 if __name__ == "__main__":
     try:
         main()
-    except Exception, ex:
-        print(ex)
+    except Exception:
+        print(traceback.format_exc())
